@@ -1,7 +1,10 @@
-;;; article.el - Special code for article style.
+;;; article.el - Special code for article style.  -*- lexical-binding: t; -*-
 
 ;;; Code:
 
+(require 'tex)
+(require 'latex)
+
 (defvar LaTeX-article-class-options
   '("a4paper" "a5paper" "b5paper" "letterpaper" "legalpaper" "executivepaper"
     "landscape" "10pt" "11pt" "12pt" "oneside" "twoside" "draft" "final"
@@ -13,9 +16,9 @@
  (lambda ()
    (LaTeX-largest-level-set "section")
    (LaTeX-add-counters "part" "section" "subsection" "subsubsection" "paragraph"
-		       "subparagraph" "figure" "table")
+                       "subparagraph" "figure" "table")
    (LaTeX-add-pagestyles "headings" "myheadings")
    (LaTeX-add-environments "abstract"))
- LaTeX-dialect)
+ TeX-dialect)
 
 ;;; article.el ends here
