@@ -1,8 +1,12 @@
-;;; epsf.el - Support for the epsf style option.
+;;; epsf.el - Support for the epsf style option.  -*- lexical-binding: t; -*-
+
+;; Copyright (C) 2013-2014, 2020 Free Software Foundation, Inc.
 
 ;; Contributed by Marc Gemis <makke@wins.uia.ac.be>
 
-;;; Code: 
+;;; Code:
+
+(require 'tex)
 
 (TeX-add-style-hook
  "epsf"
@@ -15,12 +19,12 @@
     "epsfnormal" "epsfgetbb" "other" "epsfsetgraph"
     "PsFragSpecialArgs" "epsfaux" "testit" "epsfgrab"
     "epsfllx" "epsflly" "epsfury" "epsfverbosetrue"))
- LaTeX-dialect)
+ TeX-dialect)
 
-(defun TeX-arg-epsfsize (optional &optional prompt definition)
+(defun TeX-arg-epsfsize (_optional &optional _prompt _definition)
   "Create a line that print epsf figures at a certain percentage"
   (interactive)
-  (let ((scale (read-string "Scale in percent (default 75): ")))
+  (let ((scale (TeX-read-string "Scale in percent (default 75): ")))
     (setq scale (if (zerop (length scale)) "75" scale))
     (save-excursion
       ; append #1#{scale#1}
