@@ -1,11 +1,14 @@
-;;; j-report.el - Special code for j-report style.
+;;; j-report.el - Special code for j-report style.  -*- lexical-binding: t; -*-
 
 ;;; Code:
 
+(require 'tex)
+(require 'latex)
+
 (TeX-add-style-hook
  "j-report"
  (lambda ()
    (LaTeX-largest-level-set "chapter"))
- LaTeX-dialect)
+ TeX-dialect)
 
 ;;; j-report.el ends here
