@@ -1,6 +1,6 @@
 ;;; mdframed.el --- AUCTeX style for `mdframed.sty' (v1.9b)  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2016--2021 Free Software Foundation, Inc.
+;; Copyright (C) 2016--2022 Free Software Foundation, Inc.
 
 ;; Author: Arash Esbati <arash@gnu.org>
 ;; Maintainer: auctex-devel@gnu.org
@@ -276,58 +276,45 @@ and prepends them to variable `LaTeX-mdframed-key-val-options'."
     '("mdfsetup"
       (TeX-arg-key-val (LaTeX-mdframed-key-val-options)))
 
-    '("newmdenv"
+    `("newmdenv"
       [TeX-arg-key-val (LaTeX-mdframed-key-val-options)]
-      (TeX-arg-eval
-       (lambda ()
+      ,(lambda (optional)
          (let ((env (TeX-read-string
-                     (TeX-argument-prompt nil nil "Environment"))))
+                     (TeX-argument-prompt optional nil "Environment"))))
            (LaTeX-add-environments
             `(,env LaTeX-env-args [TeX-arg-key-val (LaTeX-mdframed-key-val-options)]))
            ;; Add new env's to `ispell-tex-skip-alist': skip the optional argument
            (TeX-ispell-skip-setcdr `((,env ispell-tex-arg-end 0)))
-           (format "%s" env)))))
+           (TeX-argument-insert env optional))))
 
     '("renewmdenv"
       [TeX-arg-key-val (LaTeX-mdframed-key-val-options)]
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Environment")
-                    (LaTeX-mdframed-newmdenv-list)))
+      (TeX-arg-completing-read (LaTeX-mdframed-newmdenv-list) "Environment"))
 
     '("surroundwithmdframed"
       [TeX-arg-key-val (LaTeX-mdframed-key-val-options)]
       TeX-arg-environment)
 
     '("mdflength"
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Length")
-                    '(("skipabove")
-                      ("skipbelow")
-                      ("leftmargin")
-                      ("rightmargin")
-                      ("innerleftmargin")
-                      ("innerrightmargin")
-                      ("innertopmargin")
-                      ("innerbottommargin")
-                      ("linewidth")
-                      ("innerlinewidth")
-                      ("middlelinewidth")
-                      ("outerlinewidth"))))
+      (TeX-arg-completing-read ("skipabove"       "skipbelow"
+                                "leftmargin"      "rightmargin"
+                                "innerleftmargin" "innerrightmargin"
+                                "innertopmargin"  "innerbottommargin"
+                                "linewidth"       "innerlinewidth"
+                                "middlelinewidth" "outerlinewidth")
+                               "Length"))
 
     ;; 5. Defining your own style
-    '("mdfdefinestyle"
-      (TeX-arg-eval
-       (lambda ()
+    `("mdfdefinestyle"
+      ,(lambda (optional)
          (let ((style (TeX-read-string
-                       (TeX-argument-prompt nil nil "Style name"))))
+                       (TeX-argument-prompt optional nil "Style name"))))
            (LaTeX-add-mdframed-mdfdefinestyles style)
-           (format "%s" style))))
+           (TeX-argument-insert style optional)))
       (TeX-arg-key-val (LaTeX-mdframed-key-val-options)))
 
     '("mdfapptodefinestyle"
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Style name")
-                    (LaTeX-mdframed-mdfdefinestyle-list))
+      (TeX-arg-completing-read (LaTeX-mdframed-mdfdefinestyle-list) "Style name")
       (TeX-arg-key-val (LaTeX-mdframed-key-val-options)))
 
     ;; 6.11. Title commands inside the environment
@@ -336,15 +323,14 @@ and prepends them to variable `LaTeX-mdframed-key-val-options'."
       "Subtitle")
 
     ;; 8. Theorems
-    '("newmdtheoremenv"
+    `("newmdtheoremenv"
       [TeX-arg-key-val (LaTeX-mdframed-key-val-options)]
-      (TeX-arg-eval
-       (lambda ()
+      ,(lambda (optional)
          (let ((nthm (TeX-read-string
-                      (TeX-argument-prompt nil nil "Environment"))))
+                      (TeX-argument-prompt optional nil "Environment"))))
            (LaTeX-add-environments (list nthm (vector "Heading")))
-           (format "%s" nthm))))
-      [ TeX-arg-environment "Numbered like" ]
+           (TeX-argument-insert nthm optional)))
+      [TeX-arg-environment "Numbered like"]
       t [ (TeX-arg-eval progn (if (eq (save-excursion
                                         (backward-char 2)
                                         (preceding-char))
@@ -353,16 +339,15 @@ and prepends them to variable `LaTeX-mdframed-key-val-options'."
                                 (TeX-arg-counter t "Within counter"))
                         "") ])
 
-    '("mdtheorem"
+    `("mdtheorem"
       [TeX-arg-key-val (LaTeX-mdframed-key-val-options)]
-      (TeX-arg-eval
-       (lambda ()
+      ,(lambda (optional)
          (let ((nthm (TeX-read-string
-                      (TeX-argument-prompt nil nil "Environment"))))
+                      (TeX-argument-prompt optional nil "Environment"))))
            (LaTeX-add-environments (list nthm (vector "Heading"))
                                    (list (concat nthm "*") (vector "Heading")))
-           (format "%s" nthm))))
-      [ TeX-arg-environment "Numbered like" ]
+           (TeX-argument-insert nthm optional)))
+      [TeX-arg-environment "Numbered like"]
       t [ (TeX-arg-eval progn (if (eq (save-excursion
                                         (backward-char 2)
                                         (preceding-char))
