@@ -1,6 +1,6 @@
 ;; xcolor.el --- AUCTeX style for `xcolor.sty' (v2.12)  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2016--2020 Free Software Foundation, Inc.
+;; Copyright (C) 2016--2023 Free Software Foundation, Inc.
 
 ;; Author: Arash Esbati <arash@gnu.org>
 ;; Maintainer: auctex-devel@gnu.org
@@ -26,7 +26,7 @@
 
 ;;; Commentary:
 
-;; This file adds support for `xcolor.sty' (v2.12) from 2016/05/11.
+;; This file adds support for `xcolor.sty' (v2.13) from 2021/10/31.
 ;; `xcolor.sty' is part of TeXLive.
 
 ;; `xcolor.sty' and `color.sty' share many command namens, but the
@@ -416,22 +416,17 @@ xcolor package.")
     '("providecolor" TeX-arg-xcolor-definecolor)
 
     ;; \colorlet[<type>]{<name>}[<num model>]{<color>}
-    '("colorlet"
-      [ TeX-arg-eval completing-read
-        (TeX-argument-prompt t nil "Type")
-        LaTeX-xcolor-type-color-models ]
-      (TeX-arg-eval
-       (lambda ()
+    `("colorlet"
+      [TeX-arg-completing-read LaTeX-xcolor-type-color-models "Type"]
+      ,(lambda (optional)
          (let ((xcolor (TeX-read-string
-                        (TeX-argument-prompt nil nil "Color"))))
+                        (TeX-argument-prompt optional nil "Color"))))
            (LaTeX-add-xcolor-definecolors xcolor)
-           (format "%s" xcolor))))
-      [ TeX-arg-eval completing-read
-        (TeX-argument-prompt t nil "Model")
-        (LaTeX-xcolor-color-models t) ]
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Color")
-                    (LaTeX-xcolor-definecolor-list)))
+           (TeX-argument-insert xcolor optional)))
+      [TeX-arg-completing-read ,(lambda ()
+                                  (LaTeX-xcolor-color-models t))
+                               "Model"]
+      (TeX-arg-completing-read (LaTeX-xcolor-definecolor-list) "Color"))
 
     ;; 2.5.3 Defining sets of colors
     ;; \definecolorset[<type>]{<model-list>}{<head>}{<tail>}{<set spec>}
@@ -463,6 +458,10 @@ xcolor package.")
     ;; \textcolor[<model>]{<color spec>}{<text>}
     '("textcolor" TeX-arg-xcolor "Text")
 
+    ;; \mathcolor{<name>}{<math>} or
+    ;; \mathcolor[<model>]{<color spec>}{<math>}
+    '("mathcolor" TeX-arg-xcolor "Math")
+
     ;; \pagecolor{<name>} or
     ;; \pagecolor[<model>]{<color spec>}
     '("pagecolor" TeX-arg-xcolor)
@@ -487,32 +486,20 @@ xcolor package.")
 
     ;; 2.7 Color blending
     '("blendcolors"
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Mix expr")
-                    (LaTeX-xcolor-definecolor-list)))
+      (TeX-arg-completing-read (LaTeX-xcolor-definecolor-list) "Mix expr"))
     '("blendcolors*"
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Mix expr")
-                    (LaTeX-xcolor-definecolor-list)))
+      (TeX-arg-completing-read (LaTeX-xcolor-definecolor-list) "Mix expr"))
 
     ;; 2.8 Color masks and separation
-    '("maskcolors"
-      [ TeX-arg-eval completing-read
-        (TeX-argument-prompt t nil "Model")
-        (LaTeX-xcolor-color-models t) ]
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Color")
-                    (LaTeX-xcolor-definecolor-list)))
+    `("maskcolors"
+      [TeX-arg-completing-read ,(lambda () (LaTeX-xcolor-color-models t)) "Model"]
+      (TeX-arg-completing-read (LaTeX-xcolor-definecolor-list) "Color"))
 
     ;; 2.9 Color series
     '("definecolorseries"
       "Name"
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Core model")
-                    LaTeX-xcolor-core-color-models)
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Method")
-                    '("step" "grad" "last"))
+      (TeX-arg-completing-read LaTeX-xcolor-core-color-models "Core model")
+      (TeX-arg-completing-read ("step" "grad" "last") "Method")
       [ t ] nil [ nil ] nil)
 
     '("resetcolorseries" [ "Div." ] "Name")
@@ -520,16 +507,12 @@ xcolor package.")
     ;; 2.13 Color information
     ;; \extractcolorspec{<color>}{<cmd>}
     '("extractcolorspec"
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Color")
-                    (LaTeX-xcolor-definecolor-list))
+      (TeX-arg-completing-read (LaTeX-xcolor-definecolor-list) "Color")
       (TeX-arg-define-macro "Command: \\"))
 
     ;; \extractcolorspecs{<color>}{<model-cmd>}{<color-cmd>}
     '("extractcolorspecs"
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Color")
-                    (LaTeX-xcolor-definecolor-list))
+      (TeX-arg-completing-read (LaTeX-xcolor-definecolor-list) "Color")
       (TeX-arg-define-macro "Model command: \\")
       (TeX-arg-define-macro "Color command: \\"))
 
@@ -539,15 +522,11 @@ xcolor package.")
 
     ;; 2.14 Color conversion
     ;; \convertcolorspec{<model>}{<spec>}{<target model>}{cmd>}
-    '("convertcolorspec"
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Model")
-                    (LaTeX-xcolor-color-models))
-      (TeX-arg-eval TeX-read-string
-                    (TeX-argument-prompt nil nil "Spec"))
-      (TeX-arg-eval completing-read
-                    (TeX-argument-prompt nil nil "Model")
-                    (LaTeX-xcolor-color-models t))
+    `("convertcolorspec"
+      (TeX-arg-completing-read (LaTeX-xcolor-color-models) "Model")
+      (TeX-arg-string "Spec")
+      (TeX-arg-completing-read ,(lambda () (LaTeX-xcolor-color-models t))
+                               "Target model")
       (TeX-arg-define-macro "Macro: \\")) ) ; close TeX-add-symbols
 
    ;; 2.12 Color in tables
@@ -561,38 +540,28 @@ xcolor package.")
       ;; \rowcolors[<commands>]{<row>}{<odd-row color>}{<even-row color>}
       '("rowcolors"
         (TeX-arg-conditional (y-or-n-p "With optional commands? ")
-                             ( [ t ] )
-                             (ignore))
+            ( [ t ] )
+          (ignore))
         "Row"
-        (TeX-arg-eval completing-read
-                      (TeX-argument-prompt nil nil "Odd-row color")
-                      (LaTeX-xcolor-definecolor-list))
-        (TeX-arg-eval completing-read
-                      (TeX-argument-prompt nil nil "Even-row color")
-                      (LaTeX-xcolor-definecolor-list)))
+        (TeX-arg-completing-read (LaTeX-xcolor-definecolor-list) "Odd-row color")
+        (TeX-arg-completing-read (LaTeX-xcolor-definecolor-list) "Even-row color"))
       '("rowcolors*"
         (TeX-arg-conditional (y-or-n-p "With optional commands? ")
-                             ( [ t ] )
-                             (ignore))
+            ( [ t ] )
+          (ignore))
         "Row"
-        (TeX-arg-eval completing-read
-                      (TeX-argument-prompt nil nil "Odd-row color")
-                      (LaTeX-xcolor-definecolor-list))
-        (TeX-arg-eval completing-read
-                      (TeX-argument-prompt nil nil "Even-row color")
-                      (LaTeX-xcolor-definecolor-list)))
+        (TeX-arg-completing-read (LaTeX-xcolor-definecolor-list) "Odd-row color")
+        (TeX-arg-completing-read (LaTeX-xcolor-definecolor-list) "Even-row color"))
       '("showrowcolors" 0)
       '("hiderowcolors" 0))
      (LaTeX-add-counters "rownum"))
 
    ;; 2.6.4 Color testing
    (LaTeX-add-environments
-    '("testcolors" LaTeX-env-args
-      [ TeX-arg-eval mapconcat #'identity
-        (TeX-completing-read-multiple
-         (TeX-argument-prompt t nil "Color models")
-         (LaTeX-xcolor-color-models t))
-        "," ] ))
+    `("testcolors" LaTeX-env-args
+      [TeX-arg-completing-read-multiple ,(lambda ()
+                                           (LaTeX-xcolor-color-models t))
+                                        "Color models"] ))
 
    ;; Fontification
    (when (and (featurep 'font-latex)
@@ -627,9 +596,10 @@ xcolor package.")
 
 (defvar LaTeX-xcolor-package-options
   '(;; options that determine the color driver
-    "dvips" "xdvi" "dvipdf" "dvipdfm" "dvipdfmx" "pdftex" "dvipsone"
-    "dviwindo" "emtex" "dviwin" "oztex" "textures" "pctexps" "pctexwin"
-    "pctexhp" "pctex32" "truetex" "tcidvi" "vtex" "xetex"
+    "dvipdf" "dvipdfm" "dvipdfmx" "dvips" "dvipsone" "dvisvgm"
+    "dviwin" "dviwindo" "emtex" "luatex" "oztex" "pctex32"
+    "pctexhp" "pctexps" "pctexwin" "pdftex" "tcidvi" "textures"
+    "truetex" "vtex" "xdvi" "xetex"
 
     ;; options that determine the target color model
     "natural" "rgb" "cmy" "cmyk" "hsb" "gray" "RGB" "HTML"
